@@ -1,4 +1,4 @@
-from typing import Dict, Optional, List
+from typing import Dict, Optional
 
 from robocorp_ls_core.protocols import check_implements
 from robocorp_ls_core.robotframework_log import get_logger
@@ -7,7 +7,6 @@ from robotframework_ls.impl.protocols import (
     IKeywordFound,
     IKeywordCollector,
     ICompletionContext,
-    IKeywordArg,
     ILibraryDoc,
     INode,
 )
@@ -355,9 +354,6 @@ def collect_analysis_errors(initial_completion_context):
                 ):
                     continue
 
-                if keyword_usage_info.is_argument_usage:
-                    continue
-
                 from robotframework_ls.impl.keyword_argument_analysis import (
                     KeywordArgumentAnalysis,
                 )
@@ -367,31 +363,37 @@ def collect_analysis_errors(initial_completion_context):
                     keyword_found.keyword_args
                 )
 
-                keyword_token = keyword_usage_info.node.get_token(Token.KEYWORD)
+                keyword_token = None
+                if keyword_usage_info.is_argument_usage:
+                    try:
+                        keyword_token = keyword_usage_info.node.get_tokens(
+                            Token.ARGUMENT
+                        )[keyword_usage_info.argument_usage_index]
+                    except IndexError:
+                        pass
+
+                if keyword_token is None:
+                    keyword_token = keyword_usage_info.node.get_token(Token.KEYWORD)
+
                 if keyword_token is not None:
                     for error in keyword_argument_analysis.collect_keyword_usage_errors(
                         UsageInfoForKeywordArgumentAnalysis(
-                            keyword_usage_info.node, keyword_token
+                            keyword_usage_info.node,
+                            keyword_token,
+                            keyword_usage_info.argument_usage_index,
                         )
                     ):
                         errors.append(error)
 
-                docs_without_signature = keyword_found.docs_without_signature
-                if docs_without_signature and "DEPRECATED" in docs_without_signature:
-                    import re
-
-                    matched = re.match(
-                        r"^\*DEPRECATED(.*)\*(.*)", docs_without_signature
+                if keyword_found.is_deprecated():
+                    error = create_error_from_node(
+                        keyword_usage_info.node,
+                        f"Keyword: {keyword_usage_info.name} is deprecated",
+                        tokens=[keyword_usage_info.token],
                     )
-                    if matched:
-                        error = create_error_from_node(
-                            keyword_usage_info.node,
-                            f"Keyword: {keyword_usage_info.name} is deprecated",
-                            tokens=[keyword_usage_info.token],
-                        )
-                        error.severity = DiagnosticSeverity.Hint
-                        error.tags = [DiagnosticTag.Deprecated]
-                        errors.append(error)
+                    error.severity = DiagnosticSeverity.Hint
+                    error.tags = [DiagnosticTag.Deprecated]
+                    errors.append(error)
 
             if len(errors) >= MAX_ERRORS:
                 # i.e.: Collect at most 100 errors
