@@ -11,39 +11,14 @@ from robotframework_ls.impl.protocols import (
     IKeywordArg,
     ILibraryDoc,
 )
-from typing import Tuple, Sequence, List, Dict, Optional
+from typing import Sequence, List, Dict, Optional
+from robotframework_ls.impl.text_utilities import build_keyword_docs_with_signature
+from robocorp_ls_core.lsp import MarkupContentTypedDict, MarkupKind
 
 
 log = get_logger(__name__)
 
 
-def _build_docs(keyword_name, args, docs, docs_format):
-
-    if docs_format == "markdown":
-        # Multi-line approach (it's a bit too big -- maybe as an option?)
-        # if docs_format == "markdown":
-        #     arg_docs = "  \n&nbsp;&nbsp;&nbsp;&nbsp;".join(
-        #         ("**" + (x.replace("*", "\\*") + "**") for x in args)
-        #     )
-        #     return f"**{keyword_name}**  \n&nbsp;&nbsp;&nbsp;&nbsp;{arg_docs}\n\n{docs}"
-
-        if args:
-            escaped_args = (x.replace("*", "\\*") for x in args)
-            arg_docs = f'({", ".join(escaped_args)})'
-
-        else:
-            arg_docs = ""
-        return f"**{keyword_name}{arg_docs}**\n\n{docs}"
-    else:
-        if args:
-            arg_docs = f'({", ".join(args)})'
-
-        else:
-            arg_docs = ""
-
-        return f"{keyword_name}{arg_docs}\n\n{docs}"
-
-
 class _KeywordFoundFromAst(object):
 
     __slots__ = [
@@ -101,28 +76,37 @@ class _KeywordFoundFromAst(object):
         uri = self.completion_context.doc.uri
         return os.path.splitext(os.path.basename(uris.to_fs_path(uri)))[0]
 
-    @property
-    def docs_format(self):
-        return "markdown"
+    def compute_docs_without_signature(self) -> MarkupContentTypedDict:
+        return {
+            "kind": MarkupKind.Markdown,
+            "value": self._docs_without_signature(),
+        }
+
+    def compute_docs_with_signature(self) -> MarkupContentTypedDict:
+        docs = build_keyword_docs_with_signature(
+            self.keyword_name,
+            tuple(x.original_arg for x in self.keyword_args),
+            self._docs_without_signature(),
+            "markdown",
+        )
+        return {
+            "kind": MarkupKind.Markdown,
+            "value": docs,
+        }
 
-    @property
     @instance_cache
-    def docs(self) -> str:
+    def _docs_without_signature(self) -> str:
         from robotframework_ls.impl import ast_utils
 
-        docs = ast_utils.get_documentation(self._keyword_node)
-        args = [x.original_arg for x in self.keyword_args]
+        return ast_utils.get_documentation_as_markdown(self._keyword_node)
 
-        return _build_docs(self.keyword_name, args, docs, "markdown")
-
-    @property
     @instance_cache
-    def docs_without_signature(self) -> str:
+    def is_deprecated(self) -> bool:
         from robotframework_ls.impl import ast_utils
 
-        return ast_utils.get_documentation(self._keyword_node)
+        return ast_utils.is_deprecated(self._keyword_node)
 
-    @property
+    @property  # type: ignore
     @instance_cache
     def source(self) -> str:
         from robocorp_ls_core import uris
@@ -218,13 +202,19 @@ class _KeywordFoundFromLibrary(object):
     def resource_name(self):
         return None
 
-    @property
+    @instance_cache
+    def is_deprecated(self):
+        from robotframework_ls.impl import text_utilities
+
+        return text_utilities.has_deprecated_text(self._keyword_doc.doc)
+
+    @property  # type: ignore
     @instance_cache
     def source(self):
         source = self._keyword_doc.source or self._library_doc.source
         return source
 
-    @property
+    @property  # type: ignore
     @instance_cache
     def lineno(self):
         return self._keyword_doc.lineno - 1  # i.e.: make 0-based.
@@ -257,38 +247,28 @@ class _KeywordFoundFromLibrary(object):
     def scope_end_col_offset(self) -> Optional[int]:
         return self.end_col_offset
 
-    @property
-    @instance_cache
-    def _docs_and_format(self) -> Tuple[str, str]:
+    def compute_docs_without_signature(self) -> MarkupContentTypedDict:
         from robotframework_ls.impl.robot_specbuilder import docs_and_format
 
         docs, docs_format = docs_and_format(self._keyword_doc)
-        if self.keyword_args:
-            args = [x.original_arg for x in self.keyword_args]
 
-            docs = _build_docs(self.keyword_name, args, docs, docs_format)
+        return {"kind": docs_format, "value": docs}
 
-        return docs, docs_format
-
-    @property
-    @instance_cache
-    def docs(self):
-        docs, _docs_format = self._docs_and_format
-        return docs
-
-    @property
-    @instance_cache
-    def docs_without_signature(self):
+    def compute_docs_with_signature(self) -> MarkupContentTypedDict:
         from robotframework_ls.impl.robot_specbuilder import docs_and_format
 
-        docs, _docs_format = docs_and_format(self._keyword_doc)
-        return docs
+        docs, docs_format = docs_and_format(self._keyword_doc)
+        docs = build_keyword_docs_with_signature(
+            self.keyword_name,
+            tuple(x.original_arg for x in self.keyword_args),
+            docs,
+            docs_format,
+        )
 
-    @property
-    @instance_cache
-    def docs_format(self):
-        _docs, docs_format = self._docs_and_format
-        return docs_format
+        return {
+            "kind": docs_format,
+            "value": docs,
+        }
 
     def __typecheckself__(self) -> None:
         _: IKeywordFound = check_implements(self)
@@ -407,7 +387,7 @@ def _collect_libraries_keywords(
                 keyword_name = keyword.name
                 if collector.accepts(keyword_name):
 
-                    keyword_args = []
+                    keyword_args: Sequence[IKeywordArg] = ()
                     if keyword.args:
                         keyword_args = keyword.args
 
