@@ -12,6 +12,7 @@ from robotframework_ls.impl.protocols import (
 from robocorp_ls_core.protocols import check_implements
 from typing import Optional, Sequence
 from robocorp_ls_core.lsp import RangeTypedDict
+from robocorp_ls_core.basic import isinstance_name
 
 _RF_VARIABLE = re.compile(r"([$|&|@]{[\w\s]+})")
 
@@ -260,6 +261,11 @@ def find_keyword_definition(
     from robotframework_ls.impl import ast_utils
 
     token = ast_utils.get_keyword_name_token(token_info.node, token_info.token)
+    if token is None:
+        if token_info.token.type == token_info.token.KEYWORD_NAME:
+            if isinstance_name(token_info.node, "KeywordName"):
+                token = token_info.token
+
     if token is not None:
         collector = _FindDefinitionKeywordCollector(token.value)
         collect_keywords(completion_context, collector)
