@@ -3,7 +3,7 @@ import sys
 from robotframework_ls.constants import NULL
 from robocorp_ls_core.robotframework_log import get_logger
 import threading
-from typing import Optional, Dict, Set, Iterator
+from typing import Optional, Dict, Set, Iterator, Union, Any
 from robocorp_ls_core.protocols import Sentinel, IEndPoint
 from robotframework_ls.impl.protocols import ILibraryDoc, ILibraryDocOrError
 import itertools
@@ -11,7 +11,6 @@ from robocorp_ls_core.watchdog_wrapper import IFSObserver
 from robotframework_ls.impl.robot_lsp_constants import (
     OPTION_ROBOT_LIBRARIES_LIBDOC_NEEDS_ARGS,
 )
-from functools import lru_cache
 from robotframework_ls.impl.libspec_warmup import (
     _norm_filename,
     _normfile,
@@ -20,6 +19,8 @@ from robotframework_ls.impl.libspec_warmup import (
 from pathlib import Path
 from contextlib import contextmanager
 import typing
+from robotframework_ls.impl.text_utilities import get_digest_from_string
+from robocorp_ls_core.basic import normalize_filename
 
 log = get_logger(__name__)
 
@@ -38,13 +39,6 @@ def _get_additional_info_filename(spec_filename):
     return additional_info_filename
 
 
-@lru_cache(maxsize=250)
-def _get_digest_from_string(s: str) -> str:
-    import hashlib
-
-    return hashlib.sha256(s.encode("utf-8", "replace")).hexdigest()[:8]
-
-
 @contextmanager
 def timed_acquire_mutex_for_spec_filename(spec_filename):
     from robocorp_ls_core.system_mutex import timed_acquire_mutex
@@ -62,32 +56,43 @@ def timed_acquire_mutex_for_spec_filename(spec_filename):
         ctx.__exit__()
 
 
-def _load_library_doc_and_mtime(spec_filename, obtain_mutex=True):
+def _load_library_doc_and_mtime(libspec_manager, spec_filename: str, obtain_mutex=True):
     """
     :param obtain_mutex:
         Should be False if this is part of a bigger operation that already
         has the spec_filename mutex.
     """
     from robotframework_ls.impl import robot_specbuilder
+    from robotframework_ls.impl.libspec_markdown_conversion import (
+        load_markdown_json_version,
+    )
 
+    ctx: Any
     if obtain_mutex:
         ctx = timed_acquire_mutex_for_spec_filename(spec_filename)
     else:
         ctx = NULL
     with ctx:
         # We must load it with a mutex to avoid conflicts between generating/reading.
-        builder = robot_specbuilder.SpecDocBuilder()
         try:
             mtime = os.path.getmtime(spec_filename)
-            libdoc = builder.build(spec_filename)
+            libdoc = load_markdown_json_version(libspec_manager, spec_filename, mtime)
+
+            if libdoc is None:
+                builder = robot_specbuilder.SpecDocBuilder()
+                libdoc = builder.build(spec_filename)
+                if libdoc.doc_format != "markdown":
+                    libspec_manager.schedule_conversion_to_markdown(spec_filename)
             return libdoc, mtime
         except Exception:
             log.exception("Error when loading spec info from: %s", spec_filename)
             return None
 
 
-def _load_lib_info(canonical_spec_filename, can_regenerate):
-    libdoc_and_mtime = _load_library_doc_and_mtime(canonical_spec_filename)
+def _load_lib_info(libspec_manager, canonical_spec_filename: str, can_regenerate: bool):
+    libdoc_and_mtime = _load_library_doc_and_mtime(
+        libspec_manager, canonical_spec_filename
+    )
     if libdoc_and_mtime is None:
         return None
     libdoc, mtime = libdoc_and_mtime
@@ -125,7 +130,9 @@ def _create_updated_source_to_mtime(library_doc):
     return source_to_mtime
 
 
-def _create_additional_info(spec_filename, is_builtin, obtain_mutex=True):
+def _create_additional_info(
+    libspec_manager, spec_filename, is_builtin, obtain_mutex=True
+):
     try:
         additional_info = {_IS_BUILTIN: is_builtin}
         if is_builtin:
@@ -134,7 +141,7 @@ def _create_additional_info(spec_filename, is_builtin, obtain_mutex=True):
             return additional_info
 
         library_doc_and_mtime = _load_library_doc_and_mtime(
-            spec_filename, obtain_mutex=obtain_mutex
+            libspec_manager, spec_filename, obtain_mutex=obtain_mutex
         )
         if library_doc_and_mtime is None:
             additional_info[_UNABLE_TO_LOAD] = True
@@ -169,15 +176,21 @@ def _load_spec_filename_additional_info(spec_filename):
         return {}
 
 
-def _dump_spec_filename_additional_info(spec_filename, is_builtin, obtain_mutex=True):
+def _dump_spec_filename_additional_info(
+    libspec_manager, spec_filename, is_builtin, obtain_mutex=True
+):
     """
     Creates a filename with additional information not directly available in the
     spec.
     """
+    try:
+        libspec_manager.schedule_conversion_to_markdown(spec_filename)
+    except:
+        log.exception("Error converting %s to markdown.", spec_filename)
     import json
 
     source_to_mtime = _create_additional_info(
-        spec_filename, is_builtin, obtain_mutex=obtain_mutex
+        libspec_manager, spec_filename, is_builtin, obtain_mutex=obtain_mutex
     )
     additional_info_filename = _get_additional_info_filename(spec_filename)
     with open(additional_info_filename, "w") as stream:
@@ -194,7 +207,7 @@ class _LibInfo(object):
         "_can_regenerate",
     ]
 
-    def __init__(self, library_doc, mtime, spec_filename, can_regenerate):
+    def __init__(self, library_doc: ILibraryDoc, mtime, spec_filename, can_regenerate):
         """
         :param library_doc:
         :param mtime:
@@ -341,7 +354,7 @@ class _FolderInfo(object):
                         if filename.lower().endswith(".libspec"):
                             seen_libspec_files.add(os.path.join(folder, filename))
 
-        new_libspec_filename_to_info = {}
+        new_libspec_filename_to_info: Dict[str, _LibInfo] = {}
 
         for filename in seen_libspec_files:
             filename = _norm_filename(filename)
@@ -377,30 +390,28 @@ class LibspecManager(object):
     """
 
     @classmethod
-    def get_robot_version(cls):
+    def get_robot_version(cls) -> str:
         from robotframework_ls.impl import robot_version
 
         return robot_version.get_robot_version()
 
     @classmethod
-    def get_robot_major_version(cls):
+    def get_robot_major_version(cls) -> int:
         from robotframework_ls.impl import robot_version
 
         return robot_version.get_robot_major_version()
 
     @classmethod
-    def get_internal_libspec_dir(cls):
+    def get_internal_libspec_dir(cls) -> str:
         from robotframework_ls import robot_config
 
         home = robot_config.get_robotframework_ls_home()
 
-        pyexe = sys.executable
-        if not isinstance(pyexe, bytes):
-            pyexe = pyexe.encode("utf-8")
+        pyexe: Union[bytes, str] = sys.executable
+        if isinstance(pyexe, bytes):
+            pyexe = pyexe.decode("utf-8", "replace")
 
-        import hashlib
-
-        digest = hashlib.sha256(pyexe).hexdigest()[:8]
+        digest: str = get_digest_from_string(pyexe)
 
         v = cls.get_robot_version()
 
@@ -425,6 +436,7 @@ class LibspecManager(object):
         observer: Optional[IFSObserver] = None,
         endpoint: Optional[IEndPoint] = None,
         pre_generate_libspecs: bool = False,
+        cache_libspec_dir: Optional[str] = None,
     ):
         """
         :param __internal_libspec_dir__:
@@ -438,7 +450,52 @@ class LibspecManager(object):
             dir_cache_dir
             or os.path.join(robot_config.get_robotframework_ls_home(), ".cache")
         )
+
+        self._libspec_dir = self.get_internal_libspec_dir()
+
+        self._user_libspec_dir = user_libspec_dir or os.path.join(
+            self._libspec_dir, "user"
+        )
+        self._cache_libspec_dir = cache_libspec_dir or os.path.join(
+            self._libspec_dir, "cache"
+        )
+        self._builtins_libspec_dir = (
+            builtin_libspec_dir
+            or self.get_internal_builtins_libspec_dir(self._libspec_dir)
+        )
+        log.info("User libspec dir: %s", self._user_libspec_dir)
+        log.info("Builtins libspec dir: %s", self._builtins_libspec_dir)
+        log.info("Cache libspec dir: %s", self._cache_libspec_dir)
+
+        try:
+            os.makedirs(self._user_libspec_dir)
+        except:
+            # Ignore exception if it's already created.
+            pass
+        try:
+            os.makedirs(self._builtins_libspec_dir)
+        except:
+            # Ignore exception if it's already created.
+            pass
+        try:
+            os.makedirs(self._cache_libspec_dir)
+        except:
+            # Ignore exception if it's already created.
+            pass
+
         self.pre_generate_libspecs = pre_generate_libspecs
+
+        if pre_generate_libspecs:
+            from robotframework_ls.impl.libspec_markdown_conversion import (
+                LibspecMarkdownConversion,
+            )
+
+            self.libspec_markdown_conversion: Optional[
+                LibspecMarkdownConversion
+            ] = LibspecMarkdownConversion(self)
+        else:
+            self.libspec_markdown_conversion = None
+
         self._libspec_warmup = LibspecWarmup(endpoint, dir_cache)
 
         self._libspec_failures_cache: Dict[
@@ -459,29 +516,6 @@ class LibspecManager(object):
             self._on_file_changed, timeout=0.5, extensions=(".libspec", ".py")
         )
 
-        self._libspec_dir = self.get_internal_libspec_dir()
-
-        self._user_libspec_dir = user_libspec_dir or os.path.join(
-            self._libspec_dir, "user"
-        )
-        self._builtins_libspec_dir = (
-            builtin_libspec_dir
-            or self.get_internal_builtins_libspec_dir(self._libspec_dir)
-        )
-        log.info("User libspec dir: %s", self._user_libspec_dir)
-        log.info("Builtins libspec dir: %s", self._builtins_libspec_dir)
-
-        try:
-            os.makedirs(self._user_libspec_dir)
-        except:
-            # Ignore exception if it's already created.
-            pass
-        try:
-            os.makedirs(self._builtins_libspec_dir)
-        except:
-            # Ignore exception if it's already created.
-            pass
-
         # Spec info found in the workspace
         self._workspace_folder_uri_to_folder_info: Dict[str, _FolderInfo] = {}
         self._additional_pythonpath_folder_to_folder_info: Dict[str, _FolderInfo] = {}
@@ -532,6 +566,12 @@ class LibspecManager(object):
         self._synchronize()
         log.debug("Finished initializing LibspecManager.")
 
+    def schedule_conversion_to_markdown(self, spec_filename: str):
+        if self.libspec_markdown_conversion is not None:
+            self.libspec_markdown_conversion.schedule_conversion_to_markdown(
+                spec_filename
+            )
+
     @property
     def fs_observer(self) -> IFSObserver:
         return self._fs_observer
@@ -581,12 +621,14 @@ class LibspecManager(object):
                 log.debug("Generating user/pythonpath libraries libspec.")
                 self._libspec_warmup.gen_user_libraries(self, make_unique(pre_generate))
 
-        self.synchronize_additional_pythonpath_folders()
-
     @property
     def user_libspec_dir(self):
         return self._user_libspec_dir
 
+    @property
+    def cache_libspec_dir(self):
+        return self._cache_libspec_dir
+
     def _on_file_changed(self, spec_file, folder_info_on_change_spec):
         log.debug("File change detected: %s", spec_file)
 
@@ -740,7 +782,7 @@ class LibspecManager(object):
                 if info is None:
                     info = canonical_filename_to_info[
                         canonical_spec_filename
-                    ] = _load_lib_info(canonical_spec_filename, can_regenerate)
+                    ] = _load_lib_info(self, canonical_spec_filename, can_regenerate)
 
                 # Note: we could end up yielding a library with the same name
                 # multiple times due to its scope. It's up to the caller to
@@ -834,6 +876,9 @@ class LibspecManager(object):
             # it unless verbose mode is enabled.
             log_exception = log.debug
 
+        if not libname.replace(".", "").replace("/", "").replace("\\", "").strip():
+            return f"Unable to generate libspec for: {libname}"
+
         additional_path = None
         additional_path_exists = False
 
@@ -861,10 +906,11 @@ class LibspecManager(object):
                 call = [sys.executable]
                 major_version = self.get_robot_major_version()
                 if major_version < 4:
-                    call.extend("-m robot.libdoc --format XML:HTML".split())
+                    call.extend("-m robot.libdoc --format XML".split())
                 else:
-                    # Use default values for libspec (--format XML:HTML is deprecated).
-                    call.extend("-m robot.libdoc".split())
+                    call.extend(
+                        "-m robot.libdoc --format XML --specdocformat RAW".split()
+                    )
 
                 if additional_path and additional_path_exists:
                     call.extend(["-P", os.path.normpath(additional_path)])
@@ -924,6 +970,7 @@ class LibspecManager(object):
                             try:
                                 if mtime != os.path.getmtime(libspec_filename):
                                     _dump_spec_filename_additional_info(
+                                        self,
                                         libspec_filename,
                                         is_builtin=is_builtin,
                                         obtain_mutex=False,
@@ -969,7 +1016,10 @@ class LibspecManager(object):
                         return f"Error creating libspec: {output}"
 
                     _dump_spec_filename_additional_info(
-                        libspec_filename, is_builtin=is_builtin, obtain_mutex=False
+                        self,
+                        libspec_filename,
+                        is_builtin=is_builtin,
+                        obtain_mutex=False,
                     )
                     return None
             except Exception as e:
@@ -982,6 +1032,8 @@ class LibspecManager(object):
 
     def dispose(self):
         self._file_changes_notifier.dispose()
+        if self.libspec_markdown_conversion is not None:
+            self.libspec_markdown_conversion.dispose()
 
     def _compute_libspec_filename(
         self,
@@ -999,18 +1051,18 @@ class LibspecManager(object):
         if target_file:
             if args:
                 digest = (
-                    _get_digest_from_string(target_file)
+                    get_digest_from_string(target_file)
                     + "_"
-                    + _get_digest_from_string(args)
+                    + get_digest_from_string(args)
                 )
             else:
-                digest = _get_digest_from_string(target_file)
+                digest = get_digest_from_string(target_file)
 
             libspec_filename = os.path.join(libspec_dir, digest + ".libspec")
         elif not args:
             libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
         else:
-            digest = _get_digest_from_string(args)
+            digest = get_digest_from_string(args)
             libspec_filename = os.path.join(libspec_dir, libname + digest + ".libspec")
         return libspec_filename
 
@@ -1131,7 +1183,7 @@ class LibspecManager(object):
             )
             if found_target_filename:
                 target_file = found_target_filename
-                normalized_target_file = os.path.normcase(os.path.normpath(target_file))
+                normalized_target_file = normalize_filename(target_file)
             else:
                 builtin = libname in robot_constants.STDLIBS
 
@@ -1152,16 +1204,16 @@ class LibspecManager(object):
             if target_file and lib_info._can_regenerate:
                 if args:
                     digest = (
-                        _get_digest_from_string(target_file)
+                        get_digest_from_string(target_file)
                         + "_"
-                        + _get_digest_from_string(args)
+                        + get_digest_from_string(args)
                     )
                     found = library_doc.filename.endswith(digest + ".libspec")
 
                 else:
                     found = (
                         library_doc.source
-                        and os.path.normcase(os.path.normpath(library_doc.source))
+                        and normalize_filename(library_doc.source)
                         == normalized_target_file
                     )
                     if not found:
@@ -1179,7 +1231,7 @@ class LibspecManager(object):
                         library_doc.name and library_doc.name.lower() == libname_lower
                     )
                 else:
-                    digest = _get_digest_from_string(args)
+                    digest = get_digest_from_string(args)
                     found = library_doc.filename.endswith(
                         os.path.normcase(libname + digest + ".libspec")
                     )
