@@ -1,12 +1,19 @@
 from typing import Optional, Dict
 
 from robocorp_ls_core.client_base import LanguageServerClientBase
-from robocorp_ls_core.protocols import IIdMessageMatcher, IRobotFrameworkApiClient
+from robocorp_ls_core.protocols import (
+    IIdMessageMatcher,
+    IRobotFrameworkApiClient,
+    ILanguageServerClientBase,
+)
 from robocorp_ls_core.lsp import (
     TextDocumentTypedDict,
     ResponseTypedDict,
     PositionTypedDict,
     CodeLensTypedDict,
+    CompletionItemTypedDict,
+    CompletionsResponseTypedDict,
+    CompletionResolveResponseTypedDict,
 )
 from robocorp_ls_core.basic import implements
 
@@ -24,6 +31,16 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
         self._check_process_alive()
         self._version = None
 
+        self.stats: Dict[str, int] = {}
+
+    @implements(ILanguageServerClientBase.write)
+    def write(self, contents):
+        if isinstance(contents, dict):
+            method = contents.get("method")
+            if method:
+                self.stats[method] = self.stats.get(method, 0) + 1
+        return LanguageServerClientBase.write(self, contents)
+
     def _check_process_alive(self, raise_exception=True):
         returncode = self.server_process.poll()
         if returncode is not None:
@@ -152,6 +169,20 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             )
         )
 
+    def request_monaco_resolve_completion(
+        self,
+        completion_item: CompletionItemTypedDict,
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(
+            self._build_msg(
+                "monacoResolveCompletion",
+                completion_item=completion_item,
+            )
+        )
+
     def forward(self, method_name, params):
         self._check_process_alive()
         msg_id = self.next_id()
@@ -200,7 +231,9 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             self._build_msg("keywordComplete", doc_uri=doc_uri, line=line, col=col)
         )
 
-    def request_complete_all(self, doc_uri, line, col) -> Optional[IIdMessageMatcher]:
+    def request_complete_all(
+        self, doc_uri, line, col
+    ) -> Optional[IIdMessageMatcher[CompletionsResponseTypedDict]]:
         """
         Completes: sectionName, keyword, variables
         :Note: async complete.
@@ -209,6 +242,16 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             self._build_msg("completeAll", doc_uri=doc_uri, line=line, col=col)
         )
 
+    def request_resolve_completion_item(
+        self, completion_item: CompletionItemTypedDict
+    ) -> Optional[IIdMessageMatcher[CompletionResolveResponseTypedDict]]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(
+            self._build_msg("resolveCompletionItem", completion_item=completion_item)
+        )
+
     def request_find_definition(
         self, doc_uri, line, col
     ) -> Optional[IIdMessageMatcher]:
