@@ -55,7 +55,10 @@ class _ServerApi(object):
         self._used_python_executable: Optional[str] = None
         self._used_environ: Optional[Dict[str, str]] = None
         self._server_process = None
-        self._robotframework_api_client: Optional[IRobotFrameworkApiClient] = None
+
+        from robotframework_ls.server_api.client import RobotFrameworkApiClient
+
+        self._robotframework_api_client: Optional[RobotFrameworkApiClient] = None
 
         # We have a version of the config with the settings passed overridden
         # by the settings of a given (customized) interpreter.
@@ -70,6 +73,15 @@ class _ServerApi(object):
         self._language_server_ref = language_server_ref
         self._interpreter_info: Optional[IInterpreterInfo] = None
 
+        self._last_settings_sent: Optional[dict] = None
+
+    @property
+    def stats(self) -> Optional[dict]:
+        client = self._robotframework_api_client
+        if client is not None:
+            return client.stats
+        return None
+
     def _check_in_main_thread(self):
         curr_thread = threading.current_thread()
         if self._main_thread is not curr_thread:
@@ -113,12 +125,15 @@ class _ServerApi(object):
         self._check_in_main_thread()
         was_disposed = self._check_reinitialize()
         if not was_disposed:
-            # i.e.: when the interpreter info changes, even if it kept the same
-            # interpreter, it's possible that the configuration changed.
-            self.forward(
-                "workspace/didChangeConfiguration",
-                {"settings": self._config.get_full_settings()},
-            )
+            new_settings = self._config.get_full_settings()
+            if new_settings != self._last_settings_sent:
+                self._last_settings_sent = new_settings
+                # i.e.: when the interpreter info changes, even if it kept the same
+                # interpreter, it's possible that the configuration changed.
+                self.forward(
+                    "workspace/didChangeConfiguration",
+                    {"settings": new_settings},
+                )
 
     def _check_reinitialize(self) -> bool:
         """
@@ -371,6 +386,7 @@ class _ServerApi(object):
             self._robotframework_api_client = None
             self._used_environ = None
             self._used_python_executable = None
+            self._last_settings_sent = None
 
     def request_cancel(self, message_id) -> None:
         self._check_in_main_thread()
@@ -632,9 +648,11 @@ class ServerManager(object):
 
         - auto-import completions
         - find references
+        - signature help
         - collect tests
         - workspace tokens
         - find definition
+        - code-completion
         """
         api = self._get_regular_api(doc_uri)
         if api is not None:
