@@ -233,7 +233,11 @@ class MarkupContent(_Base):
 
 
 class ParameterInformation(_Base):
-    def __init__(self, label: str, documentation: Union[str, MarkupContent] = None):
+    def __init__(
+        self,
+        label: str,
+        documentation: Optional[MarkupContentTypedDict] = None,
+    ):
         self.label = label
         self.documentation = documentation
 
@@ -242,7 +246,7 @@ class SignatureInformation(_Base):
     def __init__(
         self,
         label: str,
-        documentation: Union[str, MarkupContent] = None,
+        documentation: Optional[MarkupContentTypedDict] = None,
         parameters: List[ParameterInformation] = None,
     ):
         self.label = label
@@ -263,7 +267,7 @@ class SignatureHelp(_Base):
 
         # This isn't part of the spec (just used to internally manipulate more information).
         self.name = ""
-        self.node = None
+        self.node: Any = None
 
     def to_dict(self):
         new_dict = {}
@@ -412,6 +416,11 @@ class LocationTypedDict(TypedDict):
     range: RangeTypedDict
 
 
+class TextEditTypedDict(TypedDict):
+    range: RangeTypedDict
+    newText: str
+
+
 class SymbolInformationTypedDict(TypedDict, total=False):
     """
     :ivar location:
@@ -459,6 +468,13 @@ class MarkupContentTypedDict(TypedDict):
     value: str
 
 
+class MonacoMarkdownStringTypedDict(TypedDict, total=False):
+    value: str
+    isTrusted: bool
+    supportThemeIcons: bool
+    uris: Any
+
+
 class CompletionItemTypedDict(TypedDict, total=False):
     #
     # The label of this completion item. By default
@@ -480,7 +496,9 @@ class CompletionItemTypedDict(TypedDict, total=False):
     detail: Optional[str]
     #
     # A human-readable string that represents a doc-comment.
-    documentation: Optional[Union[str, MarkupContentTypedDict]]
+    documentation: Optional[
+        Union[str, MarkupContentTypedDict, MonacoMarkdownStringTypedDict]
+    ]
     #
     # Indicates if this item is deprecated.
     # @deprecated Use `tags` instead.
@@ -552,7 +570,7 @@ class CompletionItemTypedDict(TypedDict, total=False):
     # Additional text edits should be used to change text unrelated to the current cursor position
     # (for example adding an import statement at the top of the file if the completion item will
     # insert an unqualified type).
-    additionalTextEdits: Optional[List[TextEdit]]
+    additionalTextEdits: Optional[List[TextEditTypedDict]]
     #
     # An optional set of characters that when pressed while this completion is active will accept it first and
     # then type that character. *Note* that all commit characters should have `length=1` and that superfluous
@@ -592,6 +610,18 @@ class ResponseTypedDict(TypedDict, total=False):
     error: ResponseErrorTypedDict  # Optional
 
 
+class CompletionsResponseTypedDict(TypedDict, total=False):
+    id: Union[int, str, None]
+    result: List[CompletionItemTypedDict]
+    error: ResponseErrorTypedDict  # Optional
+
+
+class CompletionResolveResponseTypedDict(TypedDict, total=False):
+    id: Union[int, str, None]
+    result: CompletionItemTypedDict
+    error: ResponseErrorTypedDict  # Optional
+
+
 class HoverResponseTypedDict(TypedDict, total=False):
     id: Union[int, str, None]
     result: HoverTypedDict  # Optional
@@ -647,7 +677,7 @@ class DocumentSymbolTypedDict(TypedDict, total=False):
     # but everything else like comments. This information is typically used to
     # determine if the clients cursor is inside the symbol to reveal in the
     # symbol in the UI.
-    range: Range
+    range: RangeTypedDict
 
     # The range that should be selected and revealed when this symbol is being
     # picked, e.g. the name of a function. Must be contained by the `range`.
