@@ -30,6 +30,8 @@ ENV_OPTION_LSP_DEBUG_MESSAGE_MATCHERS = "LSP_DEBUG_MESSAGE_MATCHERS"
 
 ENV_OPTION_LSP_DEBUG_PROCESS_ENVIRON = "LSP_DEBUG_PROCESS_ENVIRON"
 
+ENV_OPTION_LSP_DEBUG_REMOTE_FS_MESSAGES = "LSP_DEBUG_FS_MESSAGES"
+
 
 class BaseOptions(object):
 
@@ -41,6 +43,13 @@ class BaseOptions(object):
 
     DEBUG_MESSAGE_MATCHERS = is_true_in_env(ENV_OPTION_LSP_DEBUG_MESSAGE_MATCHERS)
     DEBUG_PROCESS_ENVIRON = is_true_in_env(ENV_OPTION_LSP_DEBUG_PROCESS_ENVIRON)
+    DEBUG_REMOTE_FS_MESSAGES = is_true_in_env(ENV_OPTION_LSP_DEBUG_REMOTE_FS_MESSAGES)
+
+    HIDE_COMMAND_MESSAGES = set()
+
+    if not DEBUG_REMOTE_FS_MESSAGES:
+        HIDE_COMMAND_MESSAGES.add("ack_notify_on_any_change")
+        HIDE_COMMAND_MESSAGES.add("notify_on_any_change")
 
     def __init__(self, args=None):
         """
