@@ -21,6 +21,7 @@ from enum import Enum
 
 if typing.TYPE_CHECKING:
     # This would lead to a circular import, so, do it only when type-checking.
+    from robocorp_ls_core.callbacks import Callback
     from robocorp_ls_core.lsp import TextDocumentContentChangeEvent
     from robocorp_ls_core.lsp import HoverResponseTypedDict
     from robocorp_ls_core.lsp import ReferencesResponseTypedDict
@@ -29,8 +30,11 @@ if typing.TYPE_CHECKING:
     from robocorp_ls_core.lsp import CodeLensTypedDict
     from robocorp_ls_core.lsp import RangeTypedDict
     from robocorp_ls_core.lsp import DocumentHighlightResponseTypedDict
-    from robocorp_ls_core.callbacks import Callback
     from robocorp_ls_core.lsp import PositionTypedDict
+    from robocorp_ls_core.lsp import CompletionItemTypedDict
+    from robocorp_ls_core.lsp import CompletionsResponseTypedDict
+    from robocorp_ls_core.lsp import CompletionResolveResponseTypedDict
+    from robocorp_ls_core.lsp import TextDocumentItem
 
 # Hack so that we don't break the runtime on versions prior to Python 3.8.
 if sys.version_info[:2] < (3, 8):
@@ -155,9 +159,11 @@ class IMessageMatcher(Generic[T], Protocol):
     msg: T
 
 
-class IIdMessageMatcher(IMessageMatcher, Protocol):
+class IIdMessageMatcher(Generic[T], Protocol):
 
     message_id: str
+    event: threading.Event
+    msg: T
 
 
 COMMUNICATION_DROPPED = CommunicationDropped()
@@ -303,12 +309,19 @@ class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
         :Note: async complete.
         """
 
-    def request_complete_all(self, doc_uri, line, col) -> Optional[IIdMessageMatcher]:
+    def request_complete_all(
+        self, doc_uri, line, col
+    ) -> Optional[IIdMessageMatcher["CompletionsResponseTypedDict"]]:
         """
         Completes: sectionName, keyword, variables
         :Note: async complete.
         """
 
+    def request_resolve_completion_item(
+        self, completion_item: "CompletionItemTypedDict"
+    ) -> Optional[IIdMessageMatcher["CompletionResolveResponseTypedDict"]]:
+        pass
+
     def request_find_definition(
         self, doc_uri, line, col
     ) -> Optional[IIdMessageMatcher]:
@@ -423,7 +436,21 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def change_doc(self, uri: str, version: int, text: str):
         pass
 
-    def get_completions(self, uri: str, line: int, col: int):
+    def get_completions(
+        self, uri: str, line: int, col: int
+    ) -> "CompletionsResponseTypedDict":
+        """
+        :param uri:
+            The uri for the request.
+        :param line:
+            0-based line.
+        :param col:
+            0-based col.
+        """
+
+    def get_completions_async(
+        self, uri: str, line: int, col: int
+    ) -> Optional[IIdMessageMatcher["CompletionsResponseTypedDict"]]:
         """
         :param uri:
             The uri for the request.
@@ -488,6 +515,11 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def request_workspace_symbols(self, query: Optional[str] = None):
         pass
 
+    def request_resolve_completion(
+        self, completion_item: "CompletionItemTypedDict"
+    ) -> "CompletionResolveResponseTypedDict":
+        pass
+
     def hover(self, uri: str, line: int, col: int):
         """
         :param uri:
@@ -719,9 +751,7 @@ class IWorkspace(Protocol):
     def remove_document(self, uri: str) -> None:
         pass
 
-    def put_document(
-        self, text_document: "robocorp_ls_core.lsp.TextDocumentItem"  # type: ignore
-    ) -> IDocument:
+    def put_document(self, text_document: "TextDocumentItem") -> IDocument:
         pass
 
     def get_document(self, doc_uri: str, accept_from_file: bool) -> Optional[IDocument]:
